//+------------------------------------------------------------------+
//| 2:23 PM 3/10/2019                      segmented_calculation.mqh |
//|                             Copyright  2019, WHRoeder@yahoo.com |
//|                                        mailto:WHRoeder@yahoo.com |
//+------------------------------------------------------------------+
//| Segmented OnCalculate Support                                    |
//+------------------------------------------------------------------+
void              alert_id(string msg){
   Alert(StringFormat("%s %s %s %s", WindowExpertName(), _Symbol,
                                     as_string(PERIOD_CURRENT), msg) );
}
string            as_string(ENUM_TIMEFRAMES period){
   if(period == PERIOD_CURRENT)  period   = (ENUM_TIMEFRAMES) _Period;
   string   period_xxx  = EnumToString(period);                // PERIOD_XXX
   return StringSubstr(period_xxx, 7);                         // XXX
}
//+------------------------------------------------------------------+
//| Event Timer Functions                                            |
//|                                                                  |
//| EventSetTimer sometimes fails in OnInit, on MT4, with internal   |
//| error. So remember to try it in OnCalculate                      |
//+------------------------------------------------------------------+
class Ticker{
 public:
            Ticker(void)    : mTickerStatus(false){}               //[CTor]
 public:                                                             // Methods
   bool     enable(void){
      if(mTickerStatus != TS_ACTIVE){
         if(!EventSetTimer(1) ){
            alert_id(StringFormat("EST:%i", _LastError) );
            mTickerStatus  = TS_ERROR;
            return false;
         }
         if(mTickerStatus == TS_ERROR)	alert_id("EST: Successful");
         mTickerStatus  = TS_ACTIVE;
      }
      return true;
   }
   void     disable(void){ EventKillTimer();    mTickerStatus = false;       }
 private:                                                            // Data
      enum     TckStatus{   TS_IDLE,	TS_ERROR,	TS_ACTIVE	};
   TckStatus	mTickerStatus;
};
// class Ticker -----------------------------------------------------+
Ticker               gTicker;

class Duration{
   //{Typedef
   #define TICK_COUNT uint
      #define TICK_COUNT_MAX UINT_MAX
   //}Typedef
 public:                                                             // Methods
   void        begin(void){   myTickCount = GetTickCount();                    }
   TICK_COUNT  get_ms(void){
      // Counter is limited by the restrictions of the system timer. Time is
      // stored as an unsigned integer, so it's overfilled every 49.7 days if a
      // computer works uninterruptedly.
      long     delay = (long)GetTickCount() - (long)myTickCount;
      if(delay < 0)  delay += long(TICK_COUNT_MAX) + 1;     // GTC rolled over.
      return (TICK_COUNT)delay;
   }
 private:                                                            // Data
   TICK_COUNT  myTickCount;
};
// class Duration ---------------------------------------------------+

class SegmentedCalculation{
 public:
   void     SegmentedCalculation(void)  : mStatus(SCS_IDLE){}           //[CTor]
 public:                                                             // Methods
   bool     is_timeout(void){             // Called by OnCalculateSegment()
      if(IS_DEBUG_MODE) return false;             		// Disable while debugging
      bool  isTimeout   = _StopFlag
                        || mDuration.get_ms() > 1500; // Saw TO at 2000.
      if(isTimeout)  mStatus = SCS_PARTIAL;
      return   isTimeout;
   }
                                                      // Called by OnCalculate()
   int         on_calculate(int rates_total, int prev_calculated){
      if(mStatus == SCS_IDLE                             // Override prev_calc-
      && prev_calculated != 0)                           // ulated if not idle
         prev_calculated   = mComputed;                  // and not restarting.
      mDuration.begin();   mStatus  = SCS_IDLE;          // Assume no timeout.
      mComputed   = OnCalculateSegment(rates_total,      // Remember return
                                       prev_calculated); // value on Timeout.
      return mComputed;
   }
   void        on_timer(void){                        // Called by OnTimer()
      if(mStatus != SCS_PARTIAL) return;                 // Nothing left to do.
      mDuration.begin();   mStatus  = SCS_COMPLETE;		// Assume no timeout.
         int      rates_total = Bars(_Symbol, _Period);
      mComputed   = OnCalculateSegment(rates_total,      // Remember return
                                       mComputed);       // value on Timeout.
   }
 private:                                                            // Data
   int         mComputed;
   Duration    mDuration;
      enum     SCStatus{   SCS_IDLE,   SCS_PARTIAL,   SCS_COMPLETE   };
   SCStatus    mStatus;
};
// class SegmentedCalculation ---------------------------------------+
SegmentedCalculation gSegmentedCalculation;
#ifndef ON_CALCULATE_EXTENDED
//+------------------------------------------------------------------+
//|  Custom indicator iteration function                             |
//+------------------------------------------------------------------+
int OnCalculate(const int        rates_total,
                const int        prev_calculated,
                const datetime&  time[],
                const double&    open[],
                const double&    high[],
                const double&    low[],
                const double&    close[],
                const long&      tick_volume[],
                const long&      volume[],
                const int&       spread[]){
   if(!gTicker.start() )   return prev_calculated;          // Not in OnInit().
   return gSegmentedCalculation.on_calculate(rates_total, prev_calculated);
}
#endif
#ifndef ON_TIMER_EXTENDED
//+------------------------------------------------------------------+
//| Custom indicator repeative timer event handler                   |
//+------------------------------------------------------------------+
void  OnTimer(void){ gSegmentedCalculation.on_timer();   }
#endif
//+------------------------------------------------------------------+
//|  Custom indicator Segmented iteration function                   |
//+------------------------------------------------------------------+
// int      OnCalculateSegment(int rates_total, int prev_calculated)
//    :
//    while(iBar > LAST){  --iBar;
//       if(gSegmentedCalculation.is_timeout() )
//          return rates_total-iBar-REDRAW_BAR;
//       :
//    }  // iBar
//    //----
//    return rates_total-iBar-REDRAW_BAR_LAST;
// }  // OnCalculateSegment
//+-------------------------------------- segmented_calculation.mqh -+
